<?php

namespace Drupal\media_entity_file_redirect;

use Drupal\Core\Cache\CacheableRedirectResponse;
use Drupal\Core\Entity\EntityTypeManager;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class MediaEntityFileRedirectSubscriber.
 */
class MediaEntityFileRedirectSubscriber implements EventSubscriberInterface {

  /**
   * An entity type manager instance.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * MediaEntityFileRedirectSubscriber constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManager $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST] = 'onRequest';
    return $events;
  }

  /**
   * Subscribed to kernel request events.
   */
  public function onRequest(GetResponseEvent $event) {
    $requestAttrs = $event->getRequest()->attributes;
    if ($requestAttrs->get('_route') === 'entity.media.canonical') {
      /** @var \Drupal\media\Entity\Media $media */
      $media = $requestAttrs->get('media');

      /** @var \Drupal\media\Entity\MediaType $mediaType */
      $mediaType = $this
        ->entityTypeManager
        ->getStorage('media_type')
        ->load($media->bundle());

      if ($mediaType && $mediaType->getThirdPartySetting('media_entity_file_redirect', 'enabled', FALSE)) {
        $fid = $media->getSource()->getSourceFieldValue($media);
        if ($fid) {
          $file = $this->entityTypeManager->getStorage('file')->load($fid);
          if ($file) {
            /** @var \Drupal\file\FileInterface $file */
            $url = file_create_url($file->getFileUri());

            $response = new CacheableRedirectResponse($url);
            $response->addCacheableDependency($mediaType);
            $response->addCacheableDependency($media);
            $response->addCacheableDependency($file);

            $event->setResponse($response);
          }
        }
      }
    }
  }

}
